<?php

/*
Plugin Name: LDS Linker
Plugin URI: http://www.joeyday.org/category/lds-linker
Description: Changes any reference to the LDS Standard Works into a hyperlink pointing to scriptures.lds.org.
Version: 1.1
Author: Joey Day
Author URI: http://www.joeyday.org
*/

/* If you want to change the settings, define these variables as explained below. */

define('INCLUDE_BIBLE_REFERENCES', TRUE);
// To link Bible verses to lds.org, leave this set to TRUE.
// If you'd like to use a plugin like Scripturizer to link
// Bible verses, set this to FALSE.

/* Probably shouldn't edit anything below this line unless you know what you're doing. */

/**
 * Namespace for LDSLinker.
 */
class LDSLinker {

    /**
     * Hyperlink Standard Works references in text.
     *
     * @param data String containing the text.
     * @param callback A function object to be used to replace scripture
     * references with hyperlinks.
     * @return ldslinkifyd text.
     */
    function addLink($data, $callback) {
        $volumes = 'I+|1st|2nd|3rd|First|Second|Third|1|2|3';
        $books = '';
        if (INCLUDE_BIBLE_REFERENCES) {
            $books .= 'Genesis|Gen\.?|Exodus|Exod?|Ex\.?|Leviticus|Lev\.?|'.
                'Numbers|Num\.?|Deuteronomy|Deut\.?|'.
                'Joshua|Josh?\.?|Judges|Judg\.?|Ruth|Samuel|Sam\.?|'.
                'Sml\.?|Kings|Kgs\.?|Chronicles|Chr\.?|Chron\.?|Ezra|'.
                'Nehemiah|Neh\.?|Esther|Esth\.?|Job|'.
                'Psalms?|Ps\.?|Proverbs?|Prov\.?|Ecclesiastes|Eccl\.?|'.
                'Song of Solomon|Song\.?|Isaiah|Isa\.?|Jeremiah|Jer\.?|'.
                'Lamentations|Lam\.?|Ezekiel|Ezek\.?|Daniel|'.
                'Dan\.?|Hosea|Joel|Amos|Obadiah|Obad\.?|Jonah|'.
                'Micah|Nahum|Habakkuk|Hab\.?|Zephaniah|'.
                'Zeph\.?|Haggai|Hag\.?|Zechariah|Zech\.?|Malachi|Mal\.?|'.
                'Matthew|Matt\.?|Mark|Luke|John|Jn\.?|'.
                'Acts|Romans|Rom\.?|Corinthians|Cor\.?|'.
                'Galatians|Gal\.?|Ephesians|Eph\.?|'.
                'Philippians|Philip\.?|Colossians|Col\.?|Thessalonians|'.
                'Thes\.?|Timothy|Tim\.?|Titus|Philemon|Philem\.?|'.
                'Hebrews|Heb\.?|James|Peter|Pet\.?|Jude|'.
                'Revelation|Rev\.?|';
        }

        $books .= 'Nephi|Ne\.?|Jacob|Enos|Jarom|Omni|'.
            'Words of Mormon|W\.?\s?of?\s?M\.?|Mosiah|Alma|Helaman|Hel\.?|Mormon|'.
            'Morm\.?|Ether|Moroni|Moro\.?|Doctrine\sand\sCovenants|'.
            'Doctrine\s&\sCovenants|D\s?&\s?C|Doctrine\s&amp;\sCovenants|'.
            'D\s?&amp;\s?C|Doctrine\s&\#38;\sCovenants|D\s?&\#38;\s?C|'.
            'Joseph\sSmith\sMatthew|JS-M|Joseph\sSmith\sHistory|'.
            'JS-H|Abraham|Abr\.?|Moses|Official\sDeclaration|OD|Articles\sof\sFaith|'.
            'A\sof\sF';

        $verses  = '\d+(:\d+)?(?:\s?[-&]\s?\d+)?';

        return preg_replace_callback("/
            (?:($volumes)\s+)?
            ($books)
            \s+
            ($verses(?:\s?,\s?$verses)*)
            (?:$version)?
            /mx",
            $callback, $data);
    }
    
    /**
     * Callback function for replacing scripture references in the text to
     * HTML hyperlinks.
     *
     * @param match Regex matching result array.
     * @param link URL to prepend the reference.
     * @param title Text to be used in title=".." attribute of the link.
     */
    function addLinkCallback($match) {
        global $_scripture_link;

        $vol = $match[1];
        $bok = $match[2];
        $ver = $match[3];

        $psg = xmlentities(($vol ? "$vol " : "").$bok.($ver ? " $ver" : ""));
        $ver = preg_replace(array('/:/', '/[,&;]/', '/\s*/'),
                            array('%3A', '%2C',     ''),
                            $ver);

        // Trim off the dot at the end of book before putting it on the link.
        $bok = preg_replace('/\.$/', '', $bok);
        $link = 'http://scriptures.lds.org/query?words=';
        $link .= ($vol ? "$vol+" : "").($bok ? "$bok+" : "").$ver;
        return "<a href=\"$link\" title=\"LDS Scriptures Internet Edition\">$psg</a>";
    }

    /**
     * Linkify a text string.
     *
     * @param text Original text to be ldslinkifyd.
     * @return ldslinkifyd text.
     */
    function ldslinkify($text) {
        $callback = create_function('$match', "return LDSLinker::addLinkCallback(".'$match'.");");
        $text = preg_split("/(<a href.*?<\/a>|<code.*?<\/code>|<pre.*?<\/pre>|<.+?>)/ms", $text, -1, PREG_SPLIT_DELIM_CAPTURE);

        $newtext = array();
        foreach ($text as $chunk) {
            if (!preg_match('/<.*>/', $chunk)) {
                $chunk = LDSLinker::addLink($chunk, $callback);
            }
            $newtext[] = $chunk;
        }
        return implode('', $newtext);
    }

}

/**
 * Function to encode entities. Works like htmlentities but a little less sloppy.
 * I lifted this from an anonymous comment at http://www.php.net/htmlentities.
 * I mostly needed it so D&C references wouldn't show up as D&amp;C.
 *
 * @param string String containing text to me encoded
 * @param quote_style Flag to determine style of quotes
 * @return encoded text
 */
function xmlentities($string, $quote_style=ENT_QUOTES) {
   static $trans;
   if (!isset($trans)) {
       $trans = get_html_translation_table(HTML_ENTITIES, $quote_style);
       foreach ($trans as $key => $value)
           $trans[$key] = '&#'.ord($key).';';
       // dont translate the '&' in case it is part of &xxx;
       $trans[chr(38)] = '&';
   }
   // after the initial translation, _do_ map standalone '&' into '&#38;'
   return preg_replace("/&(?![A-Za-z]{0,4}\w{2,3};|#[0-9]{2,3};)/","&#38;" , strtr($string, $trans));
}

// If 'add_filter' function exists, then we assume that we are initialised as
// a WordPress plugin. We will try to create a new global scope function, and
// add it to filter main/comment content.
if (function_exists('add_filter')) {
    function __ldslinkify($content) {
        return LDSLinker::ldslinkify($content);
    }

    add_filter('the_content', '__ldslinkify');
    add_filter('comment_text', '__ldslinkify');
}

